package ca.mcgill.sable.abc.impact.visualizer.views;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.DrillDownAdapter;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.texteditor.AbstractTextEditor;

import ca.mcgill.sable.abc.impact.visualizer.ImpactPlugin;
import ca.mcgill.sable.abc.impact.visualizer.actions.CollapseAllAction;
import ca.mcgill.sable.abc.impact.visualizer.actions.DoubleClickActionDelegate;
import ca.mcgill.sable.abc.impact.visualizer.actions.ExpandAllAction;
import ca.mcgill.sable.abc.impact.visualizer.actions.ToggleLinkingAction;
import ca.mcgill.sable.abc.impact.visualizer.model.ImpactFactory;

public class ImpactView extends ViewPart {
	private TreeViewer viewer;
	private DrillDownAdapter drillDownAdapter;
	public static final String ID = ImpactPlugin.PLUGIN_ID + ".views.ImpactView";
	private ISelectionListener pageSelectionListener;
	private Action toggleLinkingAction, collapseAllAction, expandAllAction;
	private boolean linkingEnabled;

	/**
	 * The constructor.
	 */
	public ImpactView() {
	}

	/**
	 * This is a callback that will allow us
	 * to create the viewer and initialize it.
	 */
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		drillDownAdapter = new DrillDownAdapter(viewer);
		viewer.setContentProvider(new ImpactViewContentProvider());
		viewer.setLabelProvider(new ImpactViewLabelProvider());
		viewer.setSorter(new ImpactViewSorter());
		viewer.setInput(ImpactFactory.getTreeData());
		viewer.setAutoExpandLevel(AbstractTreeViewer.ALL_LEVELS);
		setLinkingEnabled(false);
		makeActions();
		hookContextMenu();
		hookDoubleClickAction();
		contributeToActionBars();
		hookPageSelection();
	}

	private void makeActions() {
		toggleLinkingAction = new ToggleLinkingAction(this);
		collapseAllAction = new CollapseAllAction(this);
		expandAllAction = new ExpandAllAction(this);
	}
	
	public boolean isLinkingEnabled() {
		return linkingEnabled;
	}
	
	public void setLinkingEnabled(boolean isOn) {
		linkingEnabled = isOn;
		if (isOn == false) {
			viewer.resetFilters();
		} else {
			viewer.addFilter(new ImpactViewFilter());
		}
	}
	
	public void collapseAll() {
		viewer.collapseAll();
	}
	
	public void expandAll() {
		viewer.expandAll();
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				ImpactView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(expandAllAction);
		manager.add(collapseAllAction);
		manager.add(new Separator());
		manager.add(toggleLinkingAction);
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillContextMenu(IMenuManager manager) {
		drillDownAdapter.addNavigationActions(manager);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}
	
	private void fillLocalToolBar(IToolBarManager manager) {
		drillDownAdapter.addNavigationActions(manager);
		manager.add(new Separator());
		manager.add(expandAllAction);
		manager.add(collapseAllAction);
		manager.add(new Separator());
		manager.add(toggleLinkingAction);
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void hookDoubleClickAction() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				DoubleClickActionDelegate.actOn(event);
			}
		});
	}
	
	private void hookPageSelection() {
		pageSelectionListener = new ISelectionListener() {
			public void selectionChanged(IWorkbenchPart part, ISelection selection) {
				postSelectionChanged(part, selection);
			}
		};
		getSite().getPage().addPostSelectionListener(pageSelectionListener);
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		viewer.getControl().setFocus();
	}
	
	@Override
	public void dispose() {
		super.dispose();
		if (pageSelectionListener != null) {
			getSite().getPage().removePostSelectionListener(pageSelectionListener);
		}
	}

	public void reload() {
		viewer.getTree().setRedraw(false);
		viewer.setInput(ImpactFactory.getTreeData());
		viewer.setAutoExpandLevel(AbstractTreeViewer.ALL_LEVELS);
		viewer.getTree().setRedraw(true);
	}
	
	
	public void postSelectionChanged(IWorkbenchPart part, ISelection selection) {
		if (!(part instanceof AbstractTextEditor)) {
			// only want to respond to changes in selection in editors
			return;
		}
		
		if (part instanceof IEditorPart && selection instanceof ITextSelection) {
			IEditorPart editor = (IEditorPart)part;
			IFile file = ((FileEditorInput)editor.getEditorInput()).getFile();
			ITextSelection sel = (ITextSelection) selection;
			if (isLinkingEnabled()) {
				ImpactFactory.createShowData(file, sel);
				reload();
				ISelection viewSelection = new StructuredSelection(
						ImpactFactory.getSelectedNodes());
				viewer.setSelection(viewSelection, true);
			}
		}
	}
}